/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2013 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.ui.properties;

import static org.fortiss.af3.expression.ui.databinding.validate.IntToStringConverter.INT_STRING_CONVERTER;
import static org.fortiss.af3.expression.ui.databinding.validate.StringToIntConverter.STRING_INT_CONVERTER;
import static org.fortiss.tooling.kernel.ui.util.DataBindingUtils.performComplexTextBinding;
import static org.fortiss.tooling.kernel.ui.util.ObservableUtils.observeValue;

import org.conqat.ide.commons.ui.databinding.validate.TextToIntegerValidator;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.fortiss.af3.platform.raspberry.model.ActuatorPWM;
import org.fortiss.af3.platform.raspberry.model.RaspberryPiModelPackage;
import org.fortiss.tooling.kernel.ui.extension.base.PropertySectionBase;

/**
 * Property section for {@link ActuatorPWM}s.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 9E768B733BD37201F97399293FA0DFA7
 */
public class ActuatorPWMPropertySection extends PropertySectionBase {
	/** The actuator. */
	private ActuatorPWM actuator;
	/** The PWM channel widget. */
	private Text pwmChannelText;

	/** {@inheritDoc} */
	@Override
	public void createControls(Composite parent, TabbedPropertySheetPage tabbedPropertySheetPage) {
		super.createControls(parent, tabbedPropertySheetPage);
		pwmChannelText = createDecoratedText("PWM Channel (0, 1)");
	}

	/** {@inheritDoc} */
	@Override
	protected void setSectionInput(Object input) {
		actuator = (ActuatorPWM)input;
	}

	/** {@inheritDoc} */
	@Override
	public void refresh() {
		super.refresh();
		IObservableValue modelObservable =
				observeValue(actuator, RaspberryPiModelPackage.Literals.ACTUATOR_PWM__CHANNEL_ID);
		performComplexTextBinding(dbc, pwmChannelText, modelObservable, INT_STRING_CONVERTER,
				STRING_INT_CONVERTER, TextToIntegerValidator.INSTANCE,
				new BoundedIntPositiveZeroValidator(0, 1));
	}
}
