/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2017 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.generator.templates;

import static org.fortiss.af3.generator.common.AF3GeneratorCommonActivator.getDefault;
import static org.fortiss.af3.generator.common.utils.SourceModelElementFactory.createStaticContentSourceUnit;
import static org.fortiss.tooling.kernel.utils.LoggingUtils.error;

import java.io.IOException;
import java.io.InputStreamReader;

import org.antlr.stringtemplate.StringTemplate;
import org.antlr.stringtemplate.StringTemplateGroup;
import org.antlr.stringtemplate.language.DefaultTemplateLexer;
import org.fortiss.af3.generator.common.model.source.AbstractUnit;
import org.fortiss.af3.generator.common.model.source.StaticContentSourceUnit;

/**
 * Templates used by the RasPi C code generator.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public final class RasPiCTemplates {
	/** Returns the '.project' file used for Eclipse project. */
	public static AbstractUnit getEclipseProjectFile(String projectName) {
		StringTemplate template = makeTemplate("ProjectFile.stg", "ProjectFile");
		template.setAttribute("projectname", projectName);
		StaticContentSourceUnit unit =
				createStaticContentSourceUnit(".project", template.toString(), false);
		return unit;
	}

	/** Evaluates the given template. */
	protected static StringTemplate makeTemplate(String stgFile, String instance) {
		StringTemplateGroup stg = loadResourceSTG(stgFile);
		StringTemplate st = stg.getInstanceOf(instance);
		return st;
	}

	/** Loads the given template using the class loader of the given class. */
	private static StringTemplateGroup loadResourceSTG(String stgName) {
		InputStreamReader in =
				new InputStreamReader(RasPiCTemplates.class.getResourceAsStream(stgName));
		StringTemplateGroup templates = new StringTemplateGroup(in, DefaultTemplateLexer.class);
		try {
			in.close();
		} catch(IOException e) {
			error(getDefault(), "AF3 generator could not load stringtemplate " + stgName + ".", e);
		}
		return templates;
	}
}
